#include "uliface.h"
#include <stdio.h>
#include <tchar.h>
#include <assert.h>
using namespace UltraLite;
#define MAX_NAME_LEN 100
ULSqlca Sqlca;

static ul_char const * ConnectionParms =
    UL_TEXT( "UID=DBA;PWD=SQL" )
    UL_TEXT( ";DBF=tutcustomer.udb"  )
    UL_TEXT( ";schema_file=tutcustomer.usm" );

enum {	ERROR_MSG_LEN = 140,
	ERROR_CONTEXT_LEN = 20,
	ERROR_CALLBACK_BUF_LEN = 80 };

ul_error_action UL_GENNED_FN_MOD MyErrorCallBack(
/************************************************/
	SQLCA * 	sqlca,
	ul_void * 	user_data,
	ul_char * 	buffer )
{
    ul_error_action action;
    (void) user_data;
    (void) buffer;

    switch( sqlca->sqlcode ){
	// The following errors are used for flow control, 
	// and we don't want to report them here.
	case SQLE_NOTFOUND:
	case SQLE_ULTRALITE_DATABASE_NOT_FOUND:
	    // Suppress these warnings.
	    action = UL_ERROR_ACTION_DEFAULT;
	    break;
	case SQLE_CANNOT_ACCESS_SCHEMA_FILE:
	    _tprintf( _TEXT( 
			"Error %ld; UltraLite schema file not found\n" ),
		    sqlca->sqlcode ); 
	    action = UL_ERROR_ACTION_CANCEL;
	    break;
	case SQLE_COMMUNICATIONS_ERROR:
	    _tprintf( _TEXT( 
			"Error %ld: Communications error\n" ), 
		    sqlca->sqlcode ); 
	    action = UL_ERROR_ACTION_DEFAULT;
	    break;
	default:
	    _tprintf( _TEXT( 
			"Error %ld: \n" ), 
		    sqlca->sqlcode ); 
	    action = UL_ERROR_ACTION_DEFAULT;
	    break;
    }
    return action;
}

Connection * open_conn( DatabaseManager * dm )
{
    Connection * conn;

    conn = dm->OpenConnection( Sqlca, ConnectionParms );
    if( conn == NULL ) {
	if( Sqlca.GetSQLCode() == SQLE_ULTRALITE_DATABASE_NOT_FOUND ) {
	    // The database doesn't exist yet -- 
	    // create it using the schema file.
	    conn = dm->CreateAndOpenDatabase( Sqlca, ConnectionParms );
		_tprintf( _TEXT("Connected to a new database.\n") );
	}
    } else {
	_tprintf( _TEXT("Connected to an existing database.\n") );
    }
    return conn;
}

bool do_insert( Connection * conn )
{
    Table * table = conn->OpenTable( _TEXT("customer") );
    if( table == NULL ) {
	return false;
    }

    if( table->GetRowCount() == 0 ) {
	_tprintf( _TEXT("Inserting two rows.\n") );
	table->InsertBegin();
	table->Set( _TEXT("fname"), _TEXT("Penny") );
	table->Set( _TEXT("lname"), _TEXT("Stamp") );
	table->Insert();

	table->InsertBegin();
	table->Set( _TEXT("fname"), _TEXT("Gene") );
	table->Set( _TEXT("lname"), _TEXT("Poole") );
	table->Insert();

	conn->Commit();
    } else {
	_tprintf( _TEXT("The table has %lu rows\n"), table->GetRowCount() );
    }
    table->Release();
    return true;
}

bool do_select( Connection * conn )
{
    Table * table = conn->OpenTable( _TEXT("customer") );
    if( table == NULL ) {
	return false;
    }

    TableSchema * schema = table->GetSchema();
    if( schema == NULL ) {
	table->Release();
	return false;
    }

    ul_column_num id_cid    = schema->GetColumnID( _TEXT("id") );
    ul_column_num fname_cid = schema->GetColumnID( _TEXT("fname") );
    ul_column_num lname_cid = schema->GetColumnID( _TEXT("lname") );
    schema->Release();

    while( table->Next() ){
	char fname[ MAX_NAME_LEN ];
	char lname[ MAX_NAME_LEN ];

	table->Get( fname_cid ).GetString( fname, MAX_NAME_LEN );
	table->Get( lname_cid ).GetString( lname, MAX_NAME_LEN );
	_tprintf( "id=%d, name=%s %s\n", (int)table->Get( id_cid ), fname, lname );
    }
    table->Release();
    return true;
}

bool do_sync( Connection * conn )
{
  ul_synch_info    info;

  conn->InitSynchInfo( &info );
  info.stream = ULSocketStream();
  info.version = UL_TEXT( "ul_default" );
  info.user_name = UL_TEXT( "sample" );
  info.send_column_names = true;
  info.download_only = true;
  if( !conn->Synchronize( &info ) ) {
      return false;
  }
  return true;
}

int main()
{
    DatabaseManager * dm;
    Connection * conn;
    ul_char		buffer[ERROR_CALLBACK_BUF_LEN];

    Sqlca.Initialize();
    ULRegisterErrorCallback(
	    Sqlca.GetCA(), 
	    &MyErrorCallBack,
	    UL_NULL, 
	    buffer, 
	    ERROR_CALLBACK_BUF_LEN );
    dm = ULInitDatabaseManager( Sqlca );
    if( dm == UL_NULL ){
	// You may have mismatched UNICODE vs. ANSI runtimes.
	Sqlca.Finalize();
	return 1;	
    }

    conn = open_conn( dm );
    if( conn == UL_NULL ){
	dm->Shutdown( Sqlca );
	Sqlca.Finalize();
	return 1;	
    }

    // Uncomment these calls to change the operations carried out.
    //do_insert(conn); 
    //do_sync(conn); 
    //do_select(conn); 

    conn->Release();
    dm->Shutdown( Sqlca );
    Sqlca.Finalize();
    return 0;
}
